# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - maps parser

    This parser is used to embed Google Maps on your wiki pages.
    To output a single map, only a target parameter must be supplied.

    You can also set numerous options controlling view, permitted actions
    and information being displayed.

    Finally, you can list markers with descriptions to be put on map.
    Marker description is escaped and before publishing. Moin syntax
    in description will be recognized and parsed as valid.

    TODO:
    * parsing more than one line of description

    @copyright: 2008 by MoinMoin:PawelPacana
    @license: GNU GPL, see COPYING for details.
"""

import re
import json
import StringIO
from random import randint

from MoinMoin import wikiutil
from MoinMoin.parser import text_moin_wiki


def maps_settings(target=u'',
                  start=u'',
                  stop=u'',
                  zoom=13,
                  navigation_type=(u'small', u'large', u'zoom'),
                  navigation_ctrl=True,
                  map_ctrl=False,
                  scale_ctrl=False,
                  overview_ctrl=False,
                  draggable=True,
                  mousewheel_zoom=False,
                  keyboard_handler=False,
                  width=wikiutil.UnitArgument('400', float, ['px', '%'], defaultunit='px'),
                  height=wikiutil.UnitArgument('300', float, ['px', '%'], defaultunit='px')):
    """
    Initialize default parameters. The parameters are checked for wrong input.

    @param target: location to center map on, default is '0.0 0.0'
    @param start: start point for route drawing
    @param stop: stop point for route drawing
    @param zoom: default map zoom level
    @param navigation_type: type of navigation control to display, default is small
    @param navigation_ctrl: if set, show navigation controls
    @param map_ctrl: if set, show map type switcher control
    @param scale_ctrl: if set, show map scale
    @param overview_ctrl: if set, show map overview in corner
    @param draggable: if set, map is draggable with mouse cursor
    @param mousewheel_zoom: if set, map can be zoomed using mouse wheel
    @param keyboard_handler: if set, map can be controlled by keyboard
    @param width: default width of map is 400 px
    @param height: default height of map is 300 px
    """
    width = '%g%s' % width
    height = '%g%s' % height
    return locals()


class Parser(object):
    """google maps parser"""
    def __init__(self, raw, request, **kw):
        self.pagename = request.page.page_name
        self.raw = raw
        self.request = request
        self.formatter = request.formatter
        self.form = None
        self._ = request.getText
        self.map_id = "map%d" % randint(0, 100000)

        args = kw.get('format_args', '')
        self.init_settings = False
        # we use a macro definition to initialize the default init parameters
        # if a user enters a wrong parameter the failure is shown by the exception
        try:
            settings = wikiutil.invoke_extension_function(request, maps_settings, args)
            for k, v in settings.iteritems():
                setattr(self, k, v)
                # saves the state of valid input
            self.init_settings = True
        except ValueError, err:
            msg = u"maps: %s" % err.args[0]
            request.write(self.formatter.text(msg))

    def parse_markers(self):
        """parse list items containing location and description

        marker format:
         * location1 ... | description1 ...
         * location2 ... | description2 ...
        """
        rule = '^\s+\*\s*(?P<loc>[^\|]+[^\s\|])\s*\|?\s*(?P<desc>.*)'
        pattern = re.compile(rule)
        markers = []
        for line in self.raw.splitlines():
            m = pattern.match(line)
            if m:
                markers.append([wikiutil.escape(m.group('loc')), self.parse_description(m.group('desc'))])
        return markers

    def parse_description(self, description):
        """run moin syntax parser on marker description"""
        description = wikiutil.escape(description)
        buffer = StringIO.StringIO()
        moin_parser = MoinParser(description, self.request, buffer)
        moin_parser.format(self.formatter)
        return buffer.getvalue()

    def render(self, formatter):
        """render map"""
        # checks if initializing of all attributes in __init__ was done
        if not self.init_settings:
            return

        script = """
<script type="text/javascript">
    google.load("maps", "2");

    function check_raw_coords(location) {
        var c = location.split(' ');
        for (cnum in c) {
            if (isNaN(parseFloat(c[cnum]))) {
                return false;
            }
        }
        return true;
    }

    function createMarker(location, description, geocoder) {
        var use_geocoder = ! check_raw_coords(location);
        if (use_geocoder) {
            var marker = new GMarker(new GLatLng(0.0, 0.0));
        }
        else {
            coords = location.split(' ');
            var marker = new GMarker(new GLatLng(parseFloat(coords[0]), parseFloat(coords[1])));
        }
        GEvent.addListener(marker, "click", function() {
            marker.openInfoWindowHtml(description);
        });
        if (use_geocoder) {
            geocoder.getLatLng(location, function(coords) {
                marker.setLatLng(coords);
            });
        }
        return marker;
    }

    function initialize() {
        var container = document.getElementById('%(map_id)s')
        var map = new google.maps.Map2(container);
        var geocoder = new google.maps.ClientGeocoder();

        var target = %(target)s;
        if (target && check_raw_coords(target)) {
            var coords = target.split(' ');
            map.setCenter(new GLatLng(parseFloat(coords[0]), parseFloat(coords[1])), %(zoom)s);
        }
        else {
            map.setCenter(new GLatLng(0.0, 0.0), 1);
            if (target) {
                geocoder.getLatLng(target, function(coords) {
                    map.setCenter(coords, %(zoom)s);
                });
            }
        }

        var waypoints = '%(waypoints)s';
        if (waypoints) {
            var directions = new GDirections(map);
            directions.load(waypoints);
        }

        %(draggable)s ? '' : map.disableDragging();
        %(mousewheel_zoom)s ? map.enableScrollWheelZoom() : '';
        %(keyboard_handler)s ? new GKeyboardHandler(map) : '';

        if (%(navigation_ctrl)s) {
            switch (%(navigation_type)s) {
                case 'large': map.addControl(new GLargeMapControl());
                              break;
                case 'small': map.addControl(new GSmallMapControl());
                              break;
                case 'zoom': map.addControl(new GSmallZoomControl());
            }
        }
        %(map_ctrl)s ? map.addControl(new GMapTypeControl()) : '';
        %(scale_ctrl)s ? map.addControl(new GScaleControl()) : '';
        %(overview_ctrl)s ? map.addControl(new GOverviewMapControl()) : '';

        var marks_list = %(markers)s;
        for (mark in marks_list) {
            var marker = createMarker(marks_list[mark][0], marks_list[mark][1], geocoder)
            map.addOverlay(marker)
        }
        container.firstChild.nextSibling.style.fontSize = "6px";
    }
    google.setOnLoadCallback(initialize);

</script>""" % {'map_id': self.map_id,
                'target': json.write(self.target),
                'zoom': self.zoom,
                'draggable': json.write(self.draggable),
                'mousewheel_zoom': json.write(self.mousewheel_zoom),
                'keyboard_handler': json.write(self.keyboard_handler),
                'navigation_ctrl': json.write(self.navigation_ctrl),
                'navigation_type': json.write(self.navigation_type),
                'map_ctrl': json.write(self.map_ctrl),
                'scale_ctrl': json.write(self.scale_ctrl),
                'overview_ctrl': json.write(self.overview_ctrl),
                'markers': json.write(self.parse_markers()),
                'waypoints': self.start and self.stop and 'from: %s to: %s' % (self.start, self.stop),
                }

        return self.formatter.rawHTML(script)

    def format(self, formatter):
        """parser output"""
        # checks if initializing of all attributes in __init__ was done
        if self.init_settings:
            self.request.write(self.render(formatter))
            self.request.write(self.formatter.div(1, id=self.map_id, style='width: %s; height: %s;' %
                                                                            (self.width, self.height, )))
            self.request.write(self.formatter.div(0))


class MoinParser(text_moin_wiki.Parser):
    """moin syntax parser class writing to buffer"""
    def __init__(self, raw, request, buffer, **kw):
        text_moin_wiki.Parser.__init__(self, raw, request, **kw)
        self.request = buffer
